#include <stdio.h>
#include <math.h>

  /* Kvadratna jednadzba u obliku a*x^2 + b*x + c = 0. */

  /* Rjesenja kvadratne jednadzbe obicnim i Vietinim formulama,
     u tipu double, i zaokruzivanjem svih medjurezultata
     (izracunatih u double) na 5 dekadskih znacajnih znamenki. */

  /* Koristi tri funkcije:
      round_to_digits - za zaokruzivanje zadanog broja na
                        digits = 5 dekadskih znacajnih znamenki,
      kvad_double - za rjesavanje kvadratne jednadzbe u double,
      kvad_round - za rjesavanje kvadratne jednadzbe zaokruzivanjem
                   svakog medjurezultata na 5 dekadskih znacajnih znamenki. */


    /* Funckija za zaokruzivanje broja na 5 dekadskih znacajnih znamenki. */

double round_to_digits(double x)
{
    const double base = 10.0;
    const int digits = 5;

        /* Potencija base^(digits) = 10^5 = 100000. */
    const double upp_limit = pow(base, digits);

        /* Potencija base^(digits - 1) = 10^4 = 10000. */
    const double low_limit = pow(base, digits - 1);

    int sign = 1;    /* Predznak = 1 ili -1. */
    int pot = 0;     /* Potencija baze. */
    double x_round;

        /* Test na nulu (u tipu double). */
    if (x == 0.0) return 0.0;

        /* Test negativnog predznaka, dalje x = fabs(x). */
    if (x < 0.0) {
        sign = -1.0;
        x = -x;
    }

        /* Nadji potenciju baze za normalizirani oblik broja,
           tako da je  upp_limit > x >= low_limit. */

    if (x < low_limit) {
        do {
            x *= base;
            ++pot;
        }
        while (x < low_limit);
    }
    else if (x >= upp_limit) {
        do {
            x /= base;
            --pot;
        }
        while (x >= low_limit);
    }

        /* Zaokruzi razlomljeni dio broja. */

    x_round = floor(x);
    if (x - x_round >= 0.5) x_round = x_round + 1.0;    /* = ceil(x). */

        /* Test na nulu (u tipu double). */
    if (x_round == 0.0) return 0.0;

        /* Vrati zaokruzeni broj na polaznu potenciju baze. */
    if (pot > 0) {
        do {
            x_round /= base;
            --pot;
        }
        while (pot > 0);
    }
    else if (pot < 0) {
        do {
            x_round *= base;
            ++pot;
        }
        while (pot < 0);
    }

        /* Vrati zaokruzeni broj s predznakom. */
    if (sign == 1.0)
        return x_round;
    else
        return -x_round;
}


    /* Funckija za rjesavanje kvadratne jednadzbe u double. */

void kvad_double(double a, double b, double c,
                 double *px_1, double *px_2,
                 double *pv_1, double *pv_2)
{
    double diskr, a2, x_1, x_2, v_1, v_2, temp;

    diskr = sqrt(b * b - 4 * a * c);
    a2 = 2 * a;

        /* Obicne formule. */

    x_1 = (-b + diskr) / a2;
    x_2 = (-b - diskr) / a2;

        /* Za usporedbu, poredamo rastuce po apsolutnoj vrijednosti. */
    if (fabs(x_1) > fabs(x_2)) {
        temp = x_1;
        x_1 = x_2;
        x_2 = temp;
    }

        /* Prvo veci po apsolutnoj vrijednosti, drugi iz Vietine formule. */

    if (b >= 0.0)
        v_2 = (-b - diskr) / a2;
    else
        v_2 = (-b + diskr) / a2;
    v_1 = c / (v_2 * a);

        /* Vracanje kroz "varijabilne" argumente. */
    *px_1 = x_1;
    *px_2 = x_2;
    *pv_1 = v_1;
    *pv_2 = v_2;
}


    /* Funckija za rjesavanje kvadratne jednadzbe zaokruzivanjem
       svakog medjurezultata na 5 dekadskih znacajnih znamenki. */

void kvad_round(double a, double b, double c,
                double *px_1, double *px_2,
                double *pv_1, double *pv_2)
{
    double diskr, a2, x_1, x_2, v_1, v_2, temp;

        /* Zaokruzi koeficijente. */
    a = round_to_digits(a);
    b = round_to_digits(b);
    c = round_to_digits(c);

        /* diskr = sqrt(b * b - 4 * a * c); */

    diskr = round_to_digits(b * b);
    temp = round_to_digits(4 * a);
    temp = round_to_digits(temp * c);
    diskr = round_to_digits(diskr - temp);
    diskr = round_to_digits(sqrt(diskr));

        /* a2 = 2 * a; */
    a2 = round_to_digits(2 * a);


        /* Obicne formule. */

        /* x_1 = (-b + diskr) / a2; */
    x_1 = round_to_digits(-b + diskr);
    x_1 = round_to_digits(x_1 / a2);

        /* x_2 = (-b - diskr) / a2; */
    x_2 = round_to_digits(-b - diskr);
    x_2 = round_to_digits(x_2 / a2);

        /* Za usporedbu, poredamo rastuce po apsolutnoj vrijednosti. */
    if (fabs(x_1) > fabs(x_2)) {
        temp = x_1;
        x_1 = x_2;
        x_2 = temp;
    }

        /* Prvo veci po apsolutnoj vrijednosti, drugi iz Vietine formule. */

    if (b >= 0.0) {
            /* v_2 = (-b - diskr) / a2; */
        v_2 = round_to_digits(-b - diskr);
        v_2 = round_to_digits(v_2 / a2);
    }
    else {
            /* v_2 = (-b + diskr) / a2; */
        v_2 = round_to_digits(-b + diskr);
        v_2 = round_to_digits(v_2 / a2);
    }

        /* v_1 = c / (v_2 * a); */
    v_1 = round_to_digits(v_2 * a);
    v_1 = round_to_digits(c / v_1);

        /* Vracanje kroz "varijabilne" argumente. */
    *px_1 = x_1;
    *px_2 = x_2;
    *pv_1 = v_1;
    *pv_2 = v_2;
}


    /* Funckije za ispis koeficijenata i rjesenja. */

void print_coef(double a, double b, double c)
{
    printf("   Egzaktni koeficijenti kvadratne jednadzbe"
           " a*x^2 + b*x + c = 0\n");
    printf("     a = %8.5f,  b = %8.5f,  c = %8.5f\n", a, b ,c);
    return;
}

void print_roots(double x_1, double x_2)
{
    printf("   Egzaktna rjesenja (iz tipa extended)\n");
    printf("     x_1 = %21.17f\n", x_1);
    printf("     x_2 = %21.17f\n", x_2);
    return;
}

void print_roots_double(double x_1, double x_2,
                        double x_1_exact, double x_2_exact)
{
    double rel_err_1 = 0.0, rel_err_2 = 0.0;

    if (x_1_exact != 0.0)
        rel_err_1 = (x_1_exact - x_1) / x_1_exact;
    if (x_2_exact != 0.0)
        rel_err_2 = (x_2_exact - x_2) / x_2_exact;

    printf("     x_1 = %21.17f,    rel_greska = %13.5e\n", x_1, rel_err_1);
    printf("     x_2 = %21.17f,    rel_greska = %13.5e\n", x_2, rel_err_2);
    return;
}

void print_roots_round(double x_1, double x_2,
                       double x_1_exact, double x_2_exact)
{
    double rel_err_1 = 0.0, rel_err_2 = 0.0;

    if (x_1_exact != 0.0)
        rel_err_1 = (x_1_exact - x_1) / x_1_exact;
    if (x_2_exact != 0.0)
        rel_err_2 = (x_2_exact - x_2) / x_2_exact;

    printf("     x_1 = %10.6f,    rel_greska = %13.5e\n", x_1, rel_err_1);
    printf("     x_2 = %10.6f,    rel_greska = %13.5e\n", x_2, rel_err_2);
    return;
}


    /* Funckija za obradu jedne kvadratne jednadzbe. */

void kvad_jedn(double a, double b, double c,
               double x_1_exact, double x_2_exact)
{
    double x_1, x_2;    /* Izracunata rjesenja obicnim formulama. */
    double v_1, v_2;    /* Izracunata rjesenja Vietinim formulama. */

    print_coef(a, b, c);
    printf("\n");
    print_roots(x_1_exact, x_2_exact);    /* Egzaktna rjesenja. */

    printf("\n   ***** Double aritmetika\n");
    kvad_double(a, b, c, &x_1, &x_2, &v_1, &v_2);
    printf("\n   Obicne formule:\n");
    print_roots_double(x_1, x_2, x_1_exact, x_2_exact);
    printf("\n   Vietine formule:\n");
    print_roots_double(v_1, v_2, x_1_exact, x_2_exact);

    printf("\n   ***** Aritmetika s 5 dekadskih znacajnih znamenki\n");
    kvad_round(a, b, c, &x_1, &x_2, &v_1, &v_2);
    printf("\n   Obicne formule:\n");
    print_roots_round(x_1, x_2, x_1_exact, x_2_exact);
    printf("\n   Vietine formule:\n");
    print_roots_round(v_1, v_2, x_1_exact, x_2_exact);

    printf("\n");
    return;
}


    /* Glavni program. */


int main(void)
{
    double a, b, c;     /* Koeficijenti kvadratne jednadzbe. */
    double x_1, x_2;    /* Egzaktna rjesenja, izracunata u extended tipu. */


        /* Gautschijev primjer. */

    a = 1.0;
    b = -56.0;
    c = 1.0;
    x_1 =  1.78628407335548633E-0002;
    x_2 =  5.59821371592664451E+0001;

    printf("\n");
    printf(" Gautschijev primjer:\n");
    printf(" ===================\n\n");
    kvad_jedn(a, b, c, x_1, x_2);


        /* Gautschijev primjer, modifikacija 1. */

    a = 1.0;
    b = -49.0;
    c = 1.0;
    x_1 =  2.04166702127055996E-0002;
    x_2 =  4.89795833297872944E+0001;

    printf("\n");
    printf(" Gautschijev primjer, modifikacija 1:\n");
    printf(" ===================================\n\n");
    kvad_jedn(a, b, c, x_1, x_2);


        /* Gautschijev primjer, modifikacija 2. */

    a = 1.0;
    b = -490.0;
    c = 1.0;
    x_1 =  2.04082482646116796E-0003;
    x_2 =  4.89997959175173539E+0002;

    printf("\n");
    printf(" Gautschijev primjer, modifikacija 2:\n");
    printf(" ===================================\n\n");
    kvad_jedn(a, b, c, x_1, x_2);


        /* Skoro dvostruka nultocka u 2. Diskriminanta blizu nule. */
        /* Ovaj primjer NE valja, jer se oduzimanje/kracenje u diskriminanti
           dogadja u double, pa se tek onda zaokruzuje.
           Rezultati su nisu jako tocni, jer b^2 nije izracunat jako tocno,
           pa je kracenje napravilo dosta veliku relativnu gresku
              b^2 - 4ac = 16.00080001 - 16.0 = 0.00080001,
             izracunato = 16.001      - 16.0 = 0.001. */

    a = 1.0;
    b = -4.0001 ;
    c = 4.0 ;
    x_1 =  1.98590777598819763E+0000;
    x_2 =  2.01419222401180237E+0000;

    printf("\n");
    printf(" Skoro dvostruka nultocka u 2:\n");
    printf(" ============================\n\n");
    kvad_jedn(a, b, c, x_1, x_2);


        /* Skoro dvostruka nultocka u 10. Diskriminanta blizu nule. */
        /* Ovaj primjer NE valja, jer se oduzimanje/kracenje u diskriminanti
           dogadja u double, pa se tek onda zaokruzuje.
           Rezultati su vrlo tocni, jer je b^2 izracunat vrlo tocno, pa
           kracenje NIJE napravilo veliku relativnu gresku
              b^2 - 4ac = 400.040001 - 400.0 = 0.040001,
             izracunato = 400.04     - 400.0 = 0.04.
           Prva znamenka u rezultatu se ovdje nije skratila, a ne bi se
           skratila ni oduzimanjem na samo 5 znacajnih znamenki. */

    a = 1.0;
    b = -20.001 ;
    c = 100.0 ;
    x_1 =  9.90049875000781246E+0000;
    x_2 =  1.01005012499921875E+0001;

    printf("\n");
    printf(" Skoro dvostruka nultocka u 10:\n");
    printf(" =============================\n\n");
    kvad_jedn(a, b, c, x_1, x_2);

    return 0;
}
