#include <stdio.h>

  /* Test korektnosti algoritama za racunanje
     binomnog koeficijenta. */

/* ================================================================= */

  /* Funkcije za binomni koeficijent n povrh k. */

/* ================================================================= */

      /* Mnozenje unaprijed u brojniku. */

int binom_up(int n, int k)
{
    int bin_coef, j;

      /* Provjera granica i signal greske. */

    if (n < 0 || k < 0 || k > n) return 0;

      /* Smanji donji argument. */

    if (k > n - k) k = n - k;

    if (k == 0) return 1;

    bin_coef = n;
    for (j = 2; j <= k; ++j)
        bin_coef = bin_coef * (n - j + 1) / j;

    return bin_coef;
}

/* ================================================================= */

      /* Mnozenje unatrag u brojniku. */

int binom_down(int n, int k)
{
    int bin_coef, j;

      /* Provjera granica i signal greske. */

    if (n < 0 || k < 0 || k > n) return 0;

      /* Smanji donji argument. */

    if (k > n - k) k = n - k;

    if (k == 0) return 1;

    bin_coef = n - k + 1;
    for (j = 2; j <= k; ++j)
        bin_coef = bin_coef * (n - k + j) / j;

    return bin_coef;
}

/* ================================================================= */

      /* Prvo mnozi brojnik, pa dijeli nazivnik. */

int binom_bn(int n, int k)
{
    int bin_coef, j;

      /* Provjera granica i signal greske. */

    if (n < 0 || k < 0 || k > n) return 0;

      /* Smanji donji argument. */

    if (k > n - k) k = n - k;

    if (k == 0) return 1;

    bin_coef = 1;
    for (j = n; j > n - k; --j)
        bin_coef = bin_coef * j;
    for (j = 2; j <= k; ++j)
        bin_coef = bin_coef / j;

    return bin_coef;
}

/* ================================================================= */

      /* n-ti red Pascalovog trokuta iz n-1-og. */

void pascal_row(int row[], int n)
{
    int j;

      /* Provjera granica i signal greske. */

    if (n < 0) {
        row[0] = 0;
        return;
    }

    row[0] = 1;
    if (n > 0) {
        row[n] = 1;
        for (j = n - 1; j >= 1; --j)
            row[j] = row[j - 1] + row[j];
    }

    return;
}

/* ================================================================= */

  /* Glavni program racuna redove Pascalovog trokuta
     i provjerava korektnost rezultata. */

#define N_MAX 50

int main(void)
{
    int row[N_MAX], old_row[N_MAX];
    int n, k, bin_coef;
    int ok_bn = 1, ok_up = 1, ok_down = 1, ok_add = 1;

    for (n = 0; n < N_MAX; ++n) {

        for (k = 0; k <= n - 1; ++k)
            old_row[k] = row[k];
        old_row[n] = 1;

        pascal_row(row, n);


          /* Check add. */

        if (ok_add) {
            for (k = 1; k <= n / 2; ++k) {
                bin_coef = row[k];
                if (bin_coef <= old_row[k] || bin_coef <= old_row[k - 1]) {
                    printf("  Add: (%2d,%2d) = %11d <= max{ %11d, %11d }\n",
                           n, k, bin_coef, old_row[k - 1], old_row[k]);
                    ok_add = 0;
                }
            }
            if (!ok_add) break;
        }

          /* Check up. */

        if (ok_up) {
            for (k = 0; k <= n / 2; ++k) {
                bin_coef = binom_up(n, k);
                if (bin_coef != row[k]) {
                    printf("   Up: (%2d,%2d) = %11d <> %11d\n",
                           n, k, bin_coef, row[k]);
                    ok_up = 0;
                }
            }
            if (!ok_up) printf("\n");
        }

          /* Check down. */

        if (ok_down) {
            for (k = 0; k <= n / 2; ++k) {
                bin_coef = binom_down(n, k);
                if (bin_coef != row[k]) {
                    printf(" Down: (%2d,%2d) = %11d <> %11d\n",
                           n, k, bin_coef, row[k]);
                    ok_down = 0;
                }
            }
            if (!ok_down) printf("\n");
        }

          /* Check bn. */

        if (ok_bn) {
            for (k = 0; k <= n / 2; ++k) {
                bin_coef = binom_bn(n, k);
                if (bin_coef != row[k]) {
                    printf("   BN: (%2d,%2d) = %11d <> %11d\n",
                           n, k, bin_coef, row[k]);
                    ok_bn = 0;
                }
            }
            if (!ok_bn) printf("\n");
        }

    }

    return 0;
}
