  /* Vjezbe: Zadatak 10.1.8. (str. 113)

     Napisite funkciju koja dobiva prirodni broj n i provjerava
     je li n prost (vraca 1) ili ne (vraca 0).
     Napomena: sve radimo u tipu int (ignoriramo brojeve <= 0).

     Razne varijante funkcije:
       - test svih djelitelja izmedju 2 i n - 1,
       - test svih djelitelja do (int) sqrt(n),
       - test djelitelja 2, a zatim samo neparnih do (int) sqrt(n).
     Sve petlje po djeliteljima su while petlje!

     Glavni program testira sve tri funkcije za zadani broj n,
     tako da nalazi odredjeni (po redu) prosti broj veci ili jednak n.

     Dodana je stoperica (v. Prog2, zadnje predavanje) i ispis vremena.
  */

#include <stdio.h>
#include <math.h>
#include <time.h>


    /* Funkcija za procesorsko vrijeme u sekundama. */

double dsecnd (void) {
    return (double)( clock( ) ) / CLOCKS_PER_SEC;
}


    /* Tri funkcije za provjeru je li broj prost. */

int is_prime_all(int n)
{
        /* Testira sve djelitelje od 2 do n - 1. */

    int d;    /* Potencijalni djelitelj. */

        /* Brojevi <= 1 nisu prosti (ignoriramo <= 0). */
    if (n <= 1) return 0;

        /* Petlja po svim mogucim djeliteljima od n od 2 do n - 1. */
    d = 2;
    while (d < n) {
        if (n % d == 0) return 0;    /* Nije prost. */
        ++d;
    }

    return 1;
}


int is_prime_sqrt(int n)
{
        /* Testira sve djelitelje od 2 do max_d = (int) sqrt(n). */

    int d;        /* Potencijalni djelitelj. */
    int max_d;    /* Maksimalni potencijalni djelitelj. */

        /* Brojevi <= 1 nisu prosti (ignoriramo <= 0). */
    if (n <= 1) return 0;


        /* Najveci djelitelj kojeg treba provjeriti je
             max_d = (int) sqrt(n).
           Naime, ako n nije prost, onda ima dva netrivijalna djelitelja,
           a manji od njih je najvise jednak max_d. Ako takav djelitelj od n
           ne postoji, onda je n prost. */
        /* Napomena: Zaokruzivanje sqrt(n) na donje cijelo je korektno
           na cijelom tipu za n, tj. ne moza se dogoditi da dobijemo za
           jedan premalo. Posebno, za brojeve n koji su potpun kvadrat,
           uvijek dobijemo korektan cjelobrojni korijen. */
    max_d = (int) sqrt(n);

        /* Petlja po mogucim djeliteljima od n od 2 do max_d. */
    d = 2;
    while (d <= max_d) {
        if (n % d == 0) return 0;    /* Nije prost. */
        ++d;
    }

    return 1;
}


int is_prime_odd_sqrt(int n)
{
        /* Prvo testira parnost (djelitelj 2), a onda testira sve
           neparne djelitelje od 2 do max_d = (int) sqrt(n). */

    int d;        /* Potencijalni djelitelj. */
    int max_d;    /* Maksimalni potencijalni djelitelj. */

        /* Brojevi <= 1 nisu prosti (ignoriramo <= 0). */
    if (n <= 1) return 0;


        /* Testiraj parnost broja. Ako je paran, samo 2 je prost. */
    if (n % 2 == 0) return (n == 2);

        /* Najveci djelitelj kojeg treba provjeriti je
             max_d = (int) sqrt(n).
           Naime, ako n nije prost, onda ima dva netrivijalna djelitelja,
           a manji od njih je najvise jednak max_d. Ako takav djelitelj od n
           ne postoji, onda je n prost. */
        /* Napomena: Zaokruzivanje sqrt(n) na donje cijelo je korektno
           na cijelom tipu za n, tj. ne moza se dogoditi da dobijemo za
           jedan premalo. Posebno, za brojeve n koji su potpun kvadrat,
           uvijek dobijemo korektan cjelobrojni korijen. */
    max_d = (int) sqrt(n);

        /* Petlja po mogucim neparnim djeliteljima od n od 3 do max_d. */
    d = 3;
    while (d <= max_d) {
        if (n % d == 0) return 0;    /* Nije prost. */
        d += 2;
    }

    return 1;
}

    /* Glavni program za test sve tri funkcije:
       Provjerava je li n = 1000000007 prost (je) i usput mjeri potrebno
       procesorsko vrijeme za provjeru (pogledati Prog2, 12. predavanje). */

int main(void)
{
    int n = 1000000000;     /* Pocetni broj za trazenje prostih. */
    int p;                  /* Trazeni prosti broj. */
    int max_k = 10000;      /* Trazeni broj prostih brojeva >= n. */
    int k;                  /* Brojac prostih >= n. */
    double t1, t2, time;    /* Vremena za stopericu. */


    printf(" Test funkcija za provjeru je li broj prost.\n");

        /* Prva funkcija is_prime_all.
           Trazi najmanji prosti broj veci ili jednak n_poc. */

    printf("\n Funkcija  is_prime_all:\n");
    printf("   Trazim prvi prosti broj >= %d\n", n);
    p = n;

    t1 = dsecnd( );    /* Pocetak mjerenja vremena. */
    while (!is_prime_all(p)) ++p;
    t2 = dsecnd( );    /* Kraj mjerenja vremena. */
    time = t2 - t1;

    printf("   Trazeni prosti broj = %d,   vrijeme = %9.6f s\n", p, time);


        /* Druga i treca funkcija su prebrze za stopericu!
           Zato trazimo desettisuciti prosti broj veci ili jednak od n. */

        /* Druga funkcija is_prime_sqrt. */

    printf("\n Funkcija  is_prime_sqrt:\n");
    printf("   Trazim %d. prosti broj >= %d\n", max_k, n);
    p = n;
    k = 0;

    t1 = dsecnd( );    /* Pocetak mjerenja vremena. */
    while (1) {
        if (is_prime_sqrt(p)) ++k;
        if (k >= max_k) break;
        ++p;
    }
    t2 = dsecnd( );    /* Kraj mjerenja vremena. */
    time = t2 - t1;
    printf("   Trazeni prosti broj = %d,   vrijeme = %9.6f s\n", p, time);

        /* Treca funkcija is_prime_odd_sqrt. */

    printf("\n Funkcija  is_prime_odd_sqrt:\n");
    printf("   Trazim %d. prosti broj >= %d\n", max_k, n);
    p = n;
    k = 0;

    t1 = dsecnd( );    /* Pocetak mjerenja vremena. */
    while (1) {
        if (is_prime_odd_sqrt(p)) ++k;
        if (k >= max_k) break;
        ++p;
    }
    t2 = dsecnd( );    /* Kraj mjerenja vremena. */
    time = t2 - t1;
    printf("   Trazeni prosti broj = %d,   vrijeme = %9.6f s\n", p, time);

    return 0;
}
